#!/usr/bin/env python

"""
Common calendar client utilities.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.data import get_window_end, uri_items
from imiptools.dates import get_default_timezone
from imiptools.profile import Preferences

def update_attendees(obj, added, removed):

    """
    Update the attendees in 'obj' with the given 'added' and 'removed'
    attendee lists. A list is returned containing the attendees whose
    attendance should be cancelled.
    """

    to_cancel = []

    if added or removed:
        attendees = uri_items(obj.get_items("ATTENDEE") or [])
        sequence = obj.get_value("SEQUENCE")

        if removed:
            remaining = []

            for attendee, attendee_attr in attendees:
                if attendee in removed:

                    # Without a sequence number, assume that the event has not
                    # been published and that attendees can be silently removed.

                    if sequence is not None:
                        to_cancel.append((attendee, attendee_attr))
                else:
                    remaining.append((attendee, attendee_attr))

            attendees = remaining

        if added:
            for attendee in added:
                attendees.append((attendee, {"PARTSTAT" : "NEEDS-ACTION", "RSVP" : "TRUE"}))

        obj["ATTENDEE"] = attendees

    return to_cancel

class Client:

    "Common handler and manager methods."

    default_window_size = 100

    def __init__(self, user):
        self.user = user
        self.preferences = None

    def get_preferences(self):
        if not self.preferences and self.user:
            self.preferences = Preferences(self.user)
        return self.preferences

    def get_tzid(self):
        prefs = self.get_preferences()
        return prefs and prefs.get("TZID") or get_default_timezone()

    def get_window_size(self):
        prefs = self.get_preferences()
        try:
            return prefs and int(prefs.get("window_size")) or self.default_window_size
        except (TypeError, ValueError):
            return self.default_window_size

    def get_window_end(self):
        return get_window_end(self.get_tzid(), self.get_window_size())

    def is_sharing(self):
        prefs = self.get_preferences()
        return prefs and prefs.get("freebusy_sharing") == "share" or False

    def is_bundling(self):
        prefs = self.get_preferences()
        return prefs and prefs.get("freebusy_bundling") == "always" or False

    def is_notifying(self):
        prefs = self.get_preferences()
        return prefs and prefs.get("freebusy_messages") == "notify" or False

# vim: tabstop=4 expandtab shiftwidth=4
